@@ -95,12 +95,16 @@ static const char * const filter_list[] = {
 
 struct map_data {
 	struct obc_session *session;
-	DBusMessage *msg;
 	GHashTable *messages;
 	int16_t mas_instance_id;
 	uint8_t supported_message_types;
 };
 
+struct pending_request {
+	struct map_data *map;
+	DBusMessage *msg;
+};
+
 #define MAP_MSG_FLAG_PRIORITY	0x01
 #define MAP_MSG_FLAG_READ	0x02
 #define MAP_MSG_FLAG_SENT	0x04
@@ -123,32 +127,52 @@ struct map_msg {
 	char *status;
 	uint64_t attachment_size;
 	uint8_t flags;
+	char *folder;
 	GDBusPendingPropertySet pending;
 };
 
 struct map_parser {
-	struct map_data *data;
+	struct pending_request *request;
 	DBusMessageIter *iter;
 };
 
 static DBusConnection *conn = NULL;
 
+static struct pending_request *pending_request_new(struct map_data *map,
+							DBusMessage *message)
+{
+	struct pending_request *p;
+
+	p = g_new0(struct pending_request, 1);
+	p->map = map;
+	p->msg = dbus_message_ref(message);
+
+	return p;
+}
+
+static void pending_request_free(struct pending_request *p)
+{
+	dbus_message_unref(p->msg);
+
+	g_free(p);
+}
+
 static void simple_cb(struct obc_session *session,
 						struct obc_transfer *transfer,
 						GError *err, void *user_data)
 {
+	struct pending_request *request = user_data;
 	DBusMessage *reply;
-	struct map_data *map = user_data;
 
 	if (err != NULL)
-		reply = g_dbus_create_error(map->msg,
+		reply = g_dbus_create_error(request->msg,
 						ERROR_INTERFACE ".Failed",
 						"%s", err->message);
 	else
-		reply = dbus_message_new_method_return(map->msg);
+		reply = dbus_message_new_method_return(request->msg);
 
 	g_dbus_send_message(conn, reply);
-	dbus_message_unref(map->msg);
+	pending_request_free(request);
 }
 
 static DBusMessage *map_setpath(DBusConnection *connection,
@@ -156,6 +180,7 @@ static DBusMessage *map_setpath(DBusConnection *connection,
 {
 	struct map_data *map = user_data;
 	const char *folder;
+	struct pending_request *request;
 	GError *err = NULL;
 
 	if (dbus_message_get_args(message, NULL, DBUS_TYPE_STRING, &folder,
@@ -164,18 +189,19 @@ static DBusMessage *map_setpath(DBusConnection *connection,
 					ERROR_INTERFACE ".InvalidArguments",
 					NULL);
 
-	obc_session_setpath(map->session, folder, simple_cb, map, &err);
+	request = pending_request_new(map, message);
+
+	obc_session_setpath(map->session, folder, simple_cb, request, &err);
 	if (err != NULL) {
 		DBusMessage *reply;
 		reply =  g_dbus_create_error(message,
 						ERROR_INTERFACE ".Failed",
 						"%s", err->message);
 		g_error_free(err);
+		pending_request_free(request);
 		return reply;
 	}
 
-	map->msg = dbus_message_ref(message);
-
 	return NULL;
 }
 
@@ -216,7 +242,7 @@ static void folder_listing_cb(struct obc_session *session,
 						struct obc_transfer *transfer,
 						GError *err, void *user_data)
 {
-	struct map_data *map = user_data;
+	struct pending_request *request = user_data;
 	GMarkupParseContext *ctxt;
 	DBusMessage *reply;
 	DBusMessageIter iter, array;
@@ -225,7 +251,7 @@ static void folder_listing_cb(struct obc_session *session,
 	int perr;
 
 	if (err != NULL) {
-		reply = g_dbus_create_error(map->msg,
+		reply = g_dbus_create_error(request->msg,
 						ERROR_INTERFACE ".Failed",
 						"%s", err->message);
 		goto done;
@@ -233,14 +259,14 @@ static void folder_listing_cb(struct obc_session *session,
 
 	perr = obc_transfer_get_contents(transfer, &contents, &size);
 	if (perr < 0) {
-		reply = g_dbus_create_error(map->msg,
+		reply = g_dbus_create_error(request->msg,
 						ERROR_INTERFACE ".Failed",
 						"Error reading contents: %s",
 						strerror(-perr));
 		goto done;
 	}
 
-	reply = dbus_message_new_method_return(map->msg);
+	reply = dbus_message_new_method_return(request->msg);
 	if (reply == NULL)
 		return;
 
@@ -258,13 +284,14 @@ static void folder_listing_cb(struct obc_session *session,
 
 done:
 	g_dbus_send_message(conn, reply);
-	dbus_message_unref(map->msg);
+	pending_request_free(request);
 }
 
 static DBusMessage *get_folder_listing(struct map_data *map,
 							DBusMessage *message,
 							GObexApparam *apparam)
 {
+	struct pending_request *request;
 	struct obc_transfer *transfer;
 	GError *err = NULL;
 	DBusMessage *reply;
@@ -277,12 +304,16 @@ static DBusMessage *get_folder_listing(struct map_data *map,
 
 	obc_transfer_set_apparam(transfer, apparam);
 
-	if (obc_session_queue(map->session, transfer, folder_listing_cb, map,
-								&err)) {
-		map->msg = dbus_message_ref(message);
-		return NULL;
+	request = pending_request_new(map, message);
+
+	if (!obc_session_queue(map->session, transfer, folder_listing_cb,
+							request, &err)) {
+		pending_request_free(request);
+		goto fail;
 	}
 
+	return NULL;
+
 fail:
 	reply = g_dbus_create_error(message, ERROR_INTERFACE ".Failed", "%s",
 								err->message);
@@ -379,6 +410,7 @@ static void map_msg_free(void *data)
 	g_free(msg->path);
 	g_free(msg->subject);
 	g_free(msg->handle);
+	g_free(msg->folder);
 	g_free(msg->timestamp);
 	g_free(msg->sender);
 	g_free(msg->sender_address);
@@ -451,6 +483,16 @@ done:
 	msg->pending = 0;
 }
 
+static gboolean get_folder(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *data)
+{
+	struct map_msg *msg = data;
+
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &msg->folder);
+
+	return TRUE;
+}
+
 static gboolean subject_exists(const GDBusPropertyTable *property, void *data)
 {
 	struct map_msg *msg = data;
@@ -744,6 +786,7 @@ static const GDBusMethodTable map_msg_methods[] = {
 };
 
 static const GDBusPropertyTable map_msg_properties[] = {
+	{ "Folder", "s", get_folder },
 	{ "Subject", "s", get_subject, NULL, subject_exists },
 	{ "Timestamp", "s", get_timestamp, NULL, timestamp_exists },
 	{ "Sender", "s", get_sender, NULL, sender_exists },
@@ -775,6 +818,7 @@ static struct map_msg *map_msg_create(struct map_data *data, const char *handle)
 	msg->path = g_strdup_printf("%s/message%s",
 					obc_session_get_path(data->session),
 					handle);
+	msg->folder = g_strdup(obc_session_get_folder(data->session));
 
 	if (!g_dbus_register_interface(conn, msg->path, MAP_MSG_INTERFACE,
 						map_msg_methods, NULL,
@@ -790,164 +834,228 @@ static struct map_msg *map_msg_create(struct map_data *data, const char *handle)
 	return msg;
 }
 
-static void parse_subject(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_subject(struct map_msg *msg, const char *value)
 {
+	if (g_strcmp0(msg->subject, value) == 0)
+		return;
+
 	g_free(msg->subject);
 	msg->subject = g_strdup(value);
-	obex_dbus_dict_append(iter, "Subject", DBUS_TYPE_STRING, &value);
+
+	g_dbus_emit_property_changed(conn, msg->path,
+						MAP_MSG_INTERFACE, "Subject");
 }
 
-static void parse_datetime(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_datetime(struct map_msg *msg, const char *value)
 {
+	if (g_strcmp0(msg->timestamp, value) == 0)
+		return;
+
 	g_free(msg->timestamp);
 	msg->timestamp = g_strdup(value);
-	obex_dbus_dict_append(iter, "Timestamp", DBUS_TYPE_STRING, &value);
+
+	g_dbus_emit_property_changed(conn, msg->path,
+						MAP_MSG_INTERFACE, "Timestamp");
 }
 
-static void parse_sender(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_sender(struct map_msg *msg, const char *value)
 {
+	if (g_strcmp0(msg->sender, value) == 0)
+		return;
+
 	g_free(msg->sender);
 	msg->sender = g_strdup(value);
-	obex_dbus_dict_append(iter, "Sender", DBUS_TYPE_STRING, &value);
+
+	g_dbus_emit_property_changed(conn, msg->path,
+						MAP_MSG_INTERFACE, "Sender");
 }
 
-static void parse_sender_address(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_sender_address(struct map_msg *msg, const char *value)
 {
+	if (g_strcmp0(msg->sender_address, value) == 0)
+		return;
+
 	g_free(msg->sender_address);
 	msg->sender_address = g_strdup(value);
-	obex_dbus_dict_append(iter, "SenderAddress", DBUS_TYPE_STRING,
-								&value);
+
+	g_dbus_emit_property_changed(conn, msg->path,
+					MAP_MSG_INTERFACE, "SenderAddress");
 }
 
-static void parse_replyto(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_replyto(struct map_msg *msg, const char *value)
 {
+	if (g_strcmp0(msg->replyto, value) == 0)
+		return;
+
 	g_free(msg->replyto);
 	msg->replyto = g_strdup(value);
-	obex_dbus_dict_append(iter, "ReplyTo", DBUS_TYPE_STRING, &value);
+
+	g_dbus_emit_property_changed(conn, msg->path,
+						MAP_MSG_INTERFACE, "ReplyTo");
 }
 
-static void parse_recipient(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_recipient(struct map_msg *msg, const char *value)
 {
+	if (g_strcmp0(msg->recipient, value) == 0)
+		return;
+
 	g_free(msg->recipient);
 	msg->recipient = g_strdup(value);
-	obex_dbus_dict_append(iter, "Recipient", DBUS_TYPE_STRING, &value);
+
+	g_dbus_emit_property_changed(conn, msg->path,
+						MAP_MSG_INTERFACE, "Recipient");
 }
 
-static void parse_recipient_address(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_recipient_address(struct map_msg *msg, const char *value)
 {
+	if (g_strcmp0(msg->recipient_address, value) == 0)
+		return;
+
 	g_free(msg->recipient_address);
 	msg->recipient_address = g_strdup(value);
-	obex_dbus_dict_append(iter, "RecipientAddress", DBUS_TYPE_STRING,
-								&value);
+
+	g_dbus_emit_property_changed(conn, msg->path,
+					MAP_MSG_INTERFACE, "RecipientAddress");
 }
 
-static void parse_type(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_type(struct map_msg *msg, const char *value)
 {
+	if (g_strcmp0(msg->type, value) == 0)
+		return;
+
 	g_free(msg->type);
-	msg->type = g_strdup(value);
-	obex_dbus_dict_append(iter, "Type", DBUS_TYPE_STRING, &value);
+
+	if (strcasecmp(value, "SMS_GSM") == 0)
+		msg->type = g_strdup("sms-gsm");
+	else if (strcasecmp(value, "SMS_CDMA") == 0)
+		msg->type = g_strdup("sms-cdma");
+	else if (strcasecmp(value, "EMAIL") == 0)
+		msg->type = g_strdup("email");
+	else if (strcasecmp(value, "MMS") == 0)
+		msg->type = g_strdup("mms");
+	else
+		msg->type = NULL;
+
+	g_dbus_emit_property_changed(conn, msg->path,
+						MAP_MSG_INTERFACE, "Type");
 }
 
-static void parse_size(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_size(struct map_msg *msg, const char *value)
 {
-	msg->size = g_ascii_strtoll(value, NULL, 10);
-	obex_dbus_dict_append(iter, "Size", DBUS_TYPE_UINT64, &msg->size);
+	uint64_t size = g_ascii_strtoll(value, NULL, 10);
+
+	if (msg->size == size)
+		return;
+
+	msg->size = size;
+
+	g_dbus_emit_property_changed(conn, msg->path,
+						MAP_MSG_INTERFACE, "Size");
 }
 
-static void parse_text(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_text(struct map_msg *msg, const char *value)
 {
 	gboolean flag = strcasecmp(value, "no") != 0;
+	uint8_t oldflags = msg->flags;
 
 	if (flag)
 		msg->flags |= MAP_MSG_FLAG_TEXT;
 	else
 		msg->flags &= ~MAP_MSG_FLAG_TEXT;
 
-	obex_dbus_dict_append(iter, "Text", DBUS_TYPE_BOOLEAN, &flag);
+	if (msg->flags != oldflags)
+		g_dbus_emit_property_changed(conn, msg->path,
+						MAP_MSG_INTERFACE, "Text");
 }
 
-static void parse_status(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_status(struct map_msg *msg, const char *value)
 {
+	if (g_strcmp0(msg->status, value) == 0)
+		return;
+
 	g_free(msg->status);
 	msg->status = g_strdup(value);
-	obex_dbus_dict_append(iter, "Status", DBUS_TYPE_STRING, &value);
+
+	g_dbus_emit_property_changed(conn, msg->path,
+						MAP_MSG_INTERFACE, "Status");
 }
 
-static void parse_attachment_size(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_attachment_size(struct map_msg *msg, const char *value)
 {
-	msg->attachment_size = g_ascii_strtoll(value, NULL, 10);
-	obex_dbus_dict_append(iter, "AttachmentSize", DBUS_TYPE_UINT64,
-							&msg->attachment_size);
+	uint64_t attachment_size = g_ascii_strtoll(value, NULL, 10);
+
+	if (msg->attachment_size == attachment_size)
+		return;
+
+	msg->attachment_size = attachment_size;
+
+	g_dbus_emit_property_changed(conn, msg->path,
+					MAP_MSG_INTERFACE, "AttachmentSize");
 }
 
-static void parse_priority(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_priority(struct map_msg *msg, const char *value)
 {
 	gboolean flag = strcasecmp(value, "no") != 0;
+	uint8_t oldflags = msg->flags;
 
 	if (flag)
 		msg->flags |= MAP_MSG_FLAG_PRIORITY;
 	else
 		msg->flags &= ~MAP_MSG_FLAG_PRIORITY;
 
-	obex_dbus_dict_append(iter, "Priority", DBUS_TYPE_BOOLEAN, &flag);
+	if (msg->flags != oldflags)
+		g_dbus_emit_property_changed(conn, msg->path,
+						MAP_MSG_INTERFACE, "Priority");
 }
 
-static void parse_read(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_read(struct map_msg *msg, const char *value)
 {
 	gboolean flag = strcasecmp(value, "no") != 0;
+	uint8_t oldflags = msg->flags;
 
 	if (flag)
 		msg->flags |= MAP_MSG_FLAG_READ;
 	else
 		msg->flags &= ~MAP_MSG_FLAG_READ;
 
-	obex_dbus_dict_append(iter, "Read", DBUS_TYPE_BOOLEAN, &flag);
+	if (msg->flags != oldflags)
+		g_dbus_emit_property_changed(conn, msg->path,
+						MAP_MSG_INTERFACE, "Read");
 }
 
-static void parse_sent(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_sent(struct map_msg *msg, const char *value)
 {
 	gboolean flag = strcasecmp(value, "no") != 0;
+	uint8_t oldflags = msg->flags;
 
 	if (flag)
 		msg->flags |= MAP_MSG_FLAG_SENT;
 	else
 		msg->flags &= ~MAP_MSG_FLAG_SENT;
 
-	obex_dbus_dict_append(iter, "Sent", DBUS_TYPE_BOOLEAN, &flag);
+	if (msg->flags != oldflags)
+		g_dbus_emit_property_changed(conn, msg->path,
+						MAP_MSG_INTERFACE, "Sent");
 }
 
-static void parse_protected(struct map_msg *msg, const char *value,
-							DBusMessageIter *iter)
+static void parse_protected(struct map_msg *msg, const char *value)
 {
 	gboolean flag = strcasecmp(value, "no") != 0;
+	uint8_t oldflags = msg->flags;
 
 	if (flag)
 		msg->flags |= MAP_MSG_FLAG_PROTECTED;
 	else
 		msg->flags &= ~MAP_MSG_FLAG_PROTECTED;
 
-	obex_dbus_dict_append(iter, "Protected", DBUS_TYPE_BOOLEAN, &flag);
+	if (msg->flags != oldflags)
+		g_dbus_emit_property_changed(conn, msg->path,
+						MAP_MSG_INTERFACE, "Protected");
 }
 
 static struct map_msg_parser {
 	const char *name;
-	void (*func) (struct map_msg *msg, const char *value,
-							DBusMessageIter *iter);
+	void (*func) (struct map_msg *msg, const char *value);
 } msg_parsers[] = {
 		{ "subject", parse_subject },
 		{ "datetime", parse_datetime },
@@ -973,8 +1081,8 @@ static void msg_element(GMarkupParseContext *ctxt, const char *element,
 				gpointer user_data, GError **gerr)
 {
 	struct map_parser *parser = user_data;
-	struct map_data *data = parser->data;
-	DBusMessageIter entry, dict, *iter = parser->iter;
+	struct map_data *data = parser->request->map;
+	DBusMessageIter entry, *iter = parser->iter;
 	struct map_msg *msg;
 	const char *key;
 	int i;
@@ -1000,25 +1108,19 @@ static void msg_element(GMarkupParseContext *ctxt, const char *element,
 	dbus_message_iter_append_basic(&entry, DBUS_TYPE_OBJECT_PATH,
 								&msg->path);
 
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_ARRAY,
-					DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
-					DBUS_TYPE_STRING_AS_STRING
-					DBUS_TYPE_VARIANT_AS_STRING
-					DBUS_DICT_ENTRY_END_CHAR_AS_STRING,
-					&dict);
-
 	for (i = 0, key = names[i]; key; key = names[++i]) {
 		struct map_msg_parser *parser;
 
 		for (parser = msg_parsers; parser && parser->name; parser++) {
 			if (strcasecmp(key, parser->name) == 0) {
-				parser->func(msg, values[i], &dict);
+				parser->func(msg, values[i]);
 				break;
 			}
 		}
 	}
 
-	dbus_message_iter_close_container(&entry, &dict);
+	g_dbus_get_properties(conn, msg->path, MAP_MSG_INTERFACE, &entry);
+
 	dbus_message_iter_close_container(iter, &entry);
 }
 
@@ -1034,7 +1136,7 @@ static void message_listing_cb(struct obc_session *session,
 						struct obc_transfer *transfer,
 						GError *err, void *user_data)
 {
-	struct map_data *map = user_data;
+	struct pending_request *request = user_data;
 	struct map_parser *parser;
 	GMarkupParseContext *ctxt;
 	DBusMessage *reply;
@@ -1044,7 +1146,7 @@ static void message_listing_cb(struct obc_session *session,
 	int perr;
 
 	if (err != NULL) {
-		reply = g_dbus_create_error(map->msg,
+		reply = g_dbus_create_error(request->msg,
 						ERROR_INTERFACE ".Failed",
 						"%s", err->message);
 		goto done;
@@ -1052,14 +1154,14 @@ static void message_listing_cb(struct obc_session *session,
 
 	perr = obc_transfer_get_contents(transfer, &contents, &size);
 	if (perr < 0) {
-		reply = g_dbus_create_error(map->msg,
+		reply = g_dbus_create_error(request->msg,
 						ERROR_INTERFACE ".Failed",
 						"Error reading contents: %s",
 						strerror(-perr));
 		goto done;
 	}
 
-	reply = dbus_message_new_method_return(map->msg);
+	reply = dbus_message_new_method_return(request->msg);
 	if (reply == NULL)
 		return;
 
@@ -1076,7 +1178,7 @@ static void message_listing_cb(struct obc_session *session,
 					&array);
 
 	parser = g_new(struct map_parser, 1);
-	parser->data = map;
+	parser->request = request;
 	parser->iter = &array;
 
 	ctxt = g_markup_parse_context_new(&msg_parser, 0, parser, NULL);
@@ -1088,7 +1190,7 @@ static void message_listing_cb(struct obc_session *session,
 
 done:
 	g_dbus_send_message(conn, reply);
-	dbus_message_unref(map->msg);
+	pending_request_free(request);
 }
 
 static DBusMessage *get_message_listing(struct map_data *map,
@@ -1096,6 +1198,7 @@ static DBusMessage *get_message_listing(struct map_data *map,
 							const char *folder,
 							GObexApparam *apparam)
 {
+	struct pending_request *request;
 	struct obc_transfer *transfer;
 	GError *err = NULL;
 	DBusMessage *reply;
@@ -1108,12 +1211,16 @@ static DBusMessage *get_message_listing(struct map_data *map,
 
 	obc_transfer_set_apparam(transfer, apparam);
 
-	if (obc_session_queue(map->session, transfer, message_listing_cb, map,
-								&err)) {
-		map->msg = dbus_message_ref(message);
-		return NULL;
+	request = pending_request_new(map, message);
+
+	if (!obc_session_queue(map->session, transfer, message_listing_cb,
+							request, &err)) {
+		pending_request_free(request);
+		goto fail;
 	}
 
+	return NULL;
+
 fail:
 	reply = g_dbus_create_error(message, ERROR_INTERFACE ".Failed", "%s",
 								err->message);
@@ -1451,21 +1558,21 @@ static void update_inbox_cb(struct obc_session *session,
 				struct obc_transfer *transfer,
 				GError *err, void *user_data)
 {
-	struct map_data *map = user_data;
+	struct pending_request *request = user_data;
 	DBusMessage *reply;
 
 	if (err != NULL) {
-		reply = g_dbus_create_error(map->msg,
+		reply = g_dbus_create_error(request->msg,
 						ERROR_INTERFACE ".Failed",
 						"%s", err->message);
 		goto done;
 	}
 
-	reply = dbus_message_new_method_return(map->msg);
+	reply = dbus_message_new_method_return(request->msg);
 
 done:
 	g_dbus_send_message(conn, reply);
-	dbus_message_unref(map->msg);
+	pending_request_free(request);
 }
 
 static DBusMessage *map_update_inbox(DBusConnection *connection,
@@ -1476,6 +1583,7 @@ static DBusMessage *map_update_inbox(DBusConnection *connection,
 	char contents[2];
 	struct obc_transfer *transfer;
 	GError *err = NULL;
+	struct pending_request *request;
 
 	contents[0] = FILLER_BYTE;
 	contents[1] = '\0';
@@ -1486,11 +1594,13 @@ static DBusMessage *map_update_inbox(DBusConnection *connection,
 	if (transfer == NULL)
 		goto fail;
 
-	if (!obc_session_queue(map->session, transfer, update_inbox_cb,
-								map, &err))
-		goto fail;
+	request = pending_request_new(map, message);
 
-	map->msg = dbus_message_ref(message);
+	if (!obc_session_queue(map->session, transfer, update_inbox_cb,
+							request, &err)) {
+		pending_request_free(request);
+		goto fail;
+	}
 
 	return NULL;
 
