@@ -1376,14 +1376,14 @@ static void avrcp_play_item_dump(int level, struct frame *frm,
 	if (ctype > AVC_CTYPE_GENERAL_INQUIRY)
 		goto response;
 
-	if (len < 13) {
+	if (len < 11) {
 		printf("PDU Malformed\n");
 		raw_dump(level, frm);
 		return;
 	}
 
 	scope = get_u8(frm);
-	printf("Scope: 0x%02x (%s)", scope, scope2str(scope));
+	printf("Scope: 0x%02x (%s)\n", scope, scope2str(scope));
 
 	p_indent(level, frm);
 
@@ -1748,8 +1748,8 @@ static const char *foldertype2str(uint8_t type)
 
 static void avrcp_folder_item_dump(int level, struct frame *frm, uint16_t len)
 {
-	uint8_t type, playable, namelen;
-	uint16_t charset;
+	uint8_t type, playable;
+	uint16_t charset, namelen;
 	uint64_t uid;
 
 	p_indent(level, frm);
@@ -1781,8 +1781,8 @@ static void avrcp_folder_item_dump(int level, struct frame *frm, uint16_t len)
 
 	p_indent(level, frm);
 
-	namelen = get_u8(frm);
-	printf("NameLength: 0x%02x (%u)\n", namelen, namelen);
+	namelen = get_u16(frm);
+	printf("NameLength: 0x%04x (%u)\n", namelen, namelen);
 
 	p_indent(level, frm);
 
@@ -1845,8 +1845,10 @@ static void avrcp_media_element_item_dump(int level, struct frame *frm,
 								uint16_t len)
 {
 	uint64_t uid;
-	uint16_t charset;
-	uint8_t type, namelen, count;
+	uint16_t charset, namelen;
+	uint8_t type, count;
+
+	p_indent(level, frm);
 
 	if (len < 14) {
 		printf("PDU Malformed\n");
@@ -1869,8 +1871,8 @@ static void avrcp_media_element_item_dump(int level, struct frame *frm,
 
 	p_indent(level, frm);
 
-	namelen = get_u8(frm);
-	printf("NameLength: 0x%02x (%u)\n", namelen, namelen);
+	namelen = get_u16(frm);
+	printf("NameLength: 0x%04x (%u)\n", namelen, namelen);
 
 	p_indent(level, frm);
 
@@ -2106,15 +2108,15 @@ static void avrcp_search_dump(int level, struct frame *frm, uint8_t hdr,
 								uint16_t len)
 {
 	uint32_t uidcounter, items;
-	uint16_t charset;
-	uint8_t namelen, status;
+	uint16_t charset, namelen;
+	uint8_t status;
 
 	p_indent(level, frm);
 
 	if (hdr & 0x02)
 		goto response;
 
-	if (len < 3) {
+	if (len < 4) {
 		printf("PDU Malformed\n");
 		raw_dump(level, frm);
 		return;
@@ -2125,8 +2127,8 @@ static void avrcp_search_dump(int level, struct frame *frm, uint8_t hdr,
 
 	p_indent(level, frm);
 
-	namelen = get_u8(frm);
-	printf("Length: 0x%02x (%u)\n", namelen, namelen);
+	namelen = get_u16(frm);
+	printf("Length: 0x%04x (%u)\n", namelen, namelen);
 
 	p_indent(level, frm);
 
@@ -2135,6 +2137,7 @@ static void avrcp_search_dump(int level, struct frame *frm, uint8_t hdr,
 		uint8_t c = get_u8(frm);
 		printf("%1c", isprint(c) ? c : '.');
 	}
+	printf("\n");
 
 	return;
 
